//********************************************************************
//*** ResourcePool/SQL/RTSQLCommand.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLCommand.hh,v 1.6 2008/04/21 17:39:10 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RTSQLCOMMAND_HH
#define RESOURCEPOOL_SQL_RTSQLCOMMAND_HH

#include "ResourcePool/Command.hh"
#include "ResourcePool/SQL/SQLFetcher.hh"

namespace fatalmind {
namespace SQL {

// C -> Command, a concrete database dependend command like oracle::SQLExecute
// TM -> Threading model
// RTSQLC -> the corresponding RTSQL command. e.g. RTSQLExecute
template<class C, class TM, class RTSQLC>
class SQLCommandWrapper
: public C
{
    public:
        SQLCommandWrapper(RTSQLC& rtc, const std::string& sql)
        : C(sql)
        , rtc_(rtc)
        {
        }

        SQLCommandWrapper(RTSQLC& rtc, const std::string& sql, SQL::SQLFetcher& fetcher)
        : C(sql, fetcher)
        , rtc_(rtc)
        {
        }

		virtual ~SQLCommandWrapper() {
		}

    protected:
        virtual Clone* DoClone() const {
            return new SQLCommandWrapper<C, TM, RTSQLC>(*this);
        }

    private:
        RTSQLC& rtc_;
};

/*
 * Provides a common baseclass for all RT SQL Commands.
 */
template<class TM>
class RTSQLCommand
: public Command<ResourceType<TM> >
{
	public:
		virtual const Clone* getPlainSpecializedCommand() const = 0;
		// TODO, i don't like it to be public, but i nighter like to have sooo many friends (due to templates)
};

}; // namespace SQL
}; // namespace fatalmind

#endif
