//********************************************************************
//*** ResourcePool/SQL/testconfig.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: testconfig.cc,v 1.10 2008/04/12 17:02:04 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <stdlib.h>
#include "ResourcePool/SQL/SQLException.hh"

typedef fatalmind::ResourcePool<fatalmind::ResourceType::SQL> RP;

std::string getOracleLogon() {
	try {
		return std::string(getenv("FATALMIND_RESOURCEPOOL_ORACLE"));
	} catch (...) {
		throw fatalmind::SkippedException("missing environment FATALMIND_RESOURCEPOOL_ORACLE");
	}
}

std::string getMysqlLogon() {
	try {
		return std::string(getenv("FATALMIND_RESOURCEPOOL_MYSQL"));
	} catch (...) {
		throw fatalmind::SkippedException("missing environment FATALMIND_RESOURCEPOOL_MYSQL");
	}
}

RP::Factory getFactory(std::string logon) {
	return RP::Factory(logon);
}

void drop(RP& p) {
	try {
		RP::Command::SQLExecute sql(p, "drop table hirsch");
		p.execute(sql);
	} catch (fatalmind::SQL::SQLException& e) {
		// uncritical, we drop more often then required
	}
}

void init(RP& p) {
	drop(p);
	std::string create("create table hirsch (id numeric not null primary key, ukn varchar(255))");
	if (p.getFactory().getDatabaseType() == RP::Factory::MYSQLTX) {
		create.append(" ENGINE=InnoDB");
	}
	RP::Command::SQLExecute sql(p, create);
	p.execute(sql);
}

void clear(RP& p) {
	RP::Command::SQLExecute sql(p,"delete from hirsch");
	p.execute(sql);
}



