//********************************************************************
//*** ResourcePool/oracle/Resource.t.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: Resource.t.cc,v 1.21 2008/01/27 16:42:29 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/oracle/ResourcePool.hh"

#include "t/main.hh"

#include "testconfig.cc"

#include "Resource.hh"

using namespace std;
using namespace fatalmind;

typedef ResourcePool<ResourceType::oracle> RP;
void TEST_oracle_resource_logonurl()
{
	std::string logon = getConnectString();
	RP::Factory f(logon);
	RP p(f);
	oracleResource<DefaultThreadedModel >& dbh = p.get();
	cout << "sessionid: " << dbh.getSessionID() << endl;
	p.free(dbh);
}

void TEST_oracle_resource_two()
{
	RP::Factory* f = new RP::Factory(getConnectString());
	RP p(*f);
	delete f;

	oracleResource<DefaultThreadedModel >& dbh = p.get();
	oracleResource<DefaultThreadedModel >& dbh2 = p.get();
	
	assertEqual(dbh.getTxDepth(), 0);
	assertEqual(dbh2.getTxDepth(), 0);

	assertEqual(dbh.incTxDepth(), 0);
	assertEqual(dbh2.getTxDepth(), 0);

	assertEqual(dbh.getTxDepth(), 1);
	assertEqual(dbh2.getTxDepth(), 0);
	dbh.decTxDepth();

	assertEqual(dbh.getTxDepth(), 0);
	assertEqual(dbh2.getTxDepth(), 0);
	
	p.free(dbh);
	p.free(dbh2);
}


void TEST_oracle_resource_fail()
{
	RP::Factory f(getSID(), "scott", "that's hopefully not your password");
	RP p(f);
	try {
		oracleResource<DefaultThreadedModel >& dbh = p.get();
		p.free(dbh);
		assertNotReach();
	} catch (CreateException e) {
	}
}

void TEST_PERF_oracle_resource_empty_template() 
{
	RP::Factory f(getConnectString());

	for (int i = 0; i < 30; ++i) {
		RP p(f);
		oracleResource<DefaultThreadedModel >& dbh = p.get();
		assertTrue(p.free(dbh));
	}
}
