//********************************************************************
//*** mm/gc_ptr.t.cc
//*** Copyright (c) 2002-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: gc_ptr.t.cc,v 1.7 2008/04/12 17:02:04 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <memory>
#include "gc_ptr.hh"

template<class PTR>
void int__PERF_gc_ptr_defaultctor_and_dtor() {
	for (int i = 0; i < 100000; ++i) {
		PTR x;
	}	
}

template<class PTR>
void int__PERF_gc_ptr_ctor_and_dtor() {
	for (int i = 0; i < 100000; ++i) {
		PTR x(new int);
	}	
}

template<class PTR>
void int__PERF_gc_ptr_cctor_and_dtor() {
	PTR me(new int);
	for (int i = 0; i < 100000; ++i) {
		PTR x(me);
	}	
}

template<class PTR>
void int__PERF_gc_ptr_empty_cctor_and_dtor() {
	PTR me;
	for (int i = 0; i < 100000; ++i) {
		PTR x(me);
	}	
}

template<class PTR>
void int__PERF_gc_ptr_empty_reset() {
	PTR me;
	for (int i = 0; i < 100000; ++i) {
		me.reset();
	}	
}

template<class PTR>
void int__PERF_gc_ptr_reset_fill() {
	PTR me;
	for (int i = 0; i < 100000; ++i) {
		me.reset(new int);
	}	
}
template<class PTR>
void int__PERF_gc_ptr_reset_fill_reset_del() {
	PTR me;
	for (int i = 0; i < 100000; ++i) {
		me.reset(new int);
		me.reset();
	}	
}

typedef gc_ptr<int> ST;
typedef gc_ptr<int, fatalmind::MultiThreadedModel<> > MT;

class Base {
	public:
		Base() {
		};
		virtual ~Base() {
		}
};


class Derived : public Base {
	public:
		Derived(): Base() {
		}
		virtual ~Derived() {
		}
};

template<class T>
void testme(gc_ptr<T> x) {
	gc_ptr<Base> x3(x);
}

template<class T>
void testme(std::auto_ptr<T> x) {
	std::auto_ptr<Base> x3(x);
}

void TEST_gc_ptr_BaseCopy() {
	gc_ptr<Derived> d1(new Derived());
	gc_ptr<Derived> d2(d1);
	gc_ptr<Base>    b1(d2);
	testme(d2);

	std::auto_ptr<Derived> d3(new Derived());
	std::auto_ptr<Derived> d4(new Derived());
	std::auto_ptr<Base>    b3(d3);
	testme(d4);
}

void TEST_PERF_gc_ptr_defaultctor_and_dtor() {
	int__PERF_gc_ptr_defaultctor_and_dtor<ST>();
}
void TEST_PERF_gc_ptr_defaultctor_and_dtor_MT() {
	int__PERF_gc_ptr_defaultctor_and_dtor<MT>();
}


void TEST_PERF_gc_ptr_ctor_and_dtor() {
	int__PERF_gc_ptr_ctor_and_dtor<ST>();
}
void TEST_PERF_gc_ptr_ctor_and_dtor_MT() {
	int__PERF_gc_ptr_ctor_and_dtor<MT>();
}


void TEST_PERF_gc_ptr_cctor_and_dtor() {
	int__PERF_gc_ptr_cctor_and_dtor<ST>();
}
void TEST_PERF_gc_ptr_cctor_and_dtor_MT() {
	int__PERF_gc_ptr_cctor_and_dtor<MT>();
}


void TEST_PERF_gc_ptr_empty_cctor_and_dtor() {
	int__PERF_gc_ptr_empty_cctor_and_dtor<ST>();
}
void TEST_PERF_gc_ptr_empty_cctor_and_dtor_MT() {
	int__PERF_gc_ptr_empty_cctor_and_dtor<MT>();
}


void TEST_PERF_gc_ptr_empty_reset() {
	int__PERF_gc_ptr_empty_reset<ST>();
}
void TEST_PERF_gc_ptr_empty_reset_MT() {
	int__PERF_gc_ptr_empty_reset<MT>();
}


void TEST_PERF_gc_ptr_reset_fill() {
	int__PERF_gc_ptr_reset_fill<ST>();
}
void TEST_PERF_gc_ptr_reset_fill_MT() {
	int__PERF_gc_ptr_reset_fill<MT>();
}


void TEST_PERF_gc_ptr_reset_fill_reset_del() {
	int__PERF_gc_ptr_reset_fill_reset_del<ST>();
}
void TEST_PERF_gc_ptr_reset_fill_reset_del_MT() {
	int__PERF_gc_ptr_reset_fill_reset_del<MT>();
}

